//############################################################################
//
// File Name	: 'demo.c'
// Title		: example usage of mmc and fat functions
// Author		: Zoltan Gradwohl
// Date			: 28/07/2005
// Revised		: 28/07/2005
// Version		: 1.0
// Target MCU	: ATmega128 (2K Byte Internal SRAM required)
// Editor Tabs	: 4
//
//
// NOTE: This code is currently below version 1.0, and therefore is considered
// to be lacking in some functionality or documentation, or may not be fully
// tested.  Nonetheless, you can expect most functions to work.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//############################################################################

// Include Files 
#include <avr/io.h>			// include I/O definitions (port names, pin names, etc)
#include <avr/signal.h>		// include "signal" names (interrupt names)
#include <avr/interrupt.h>	// include interrupt support
#include <string.h>			// include string support

#include "global.h"		// include our global settings
#include "uart.h"		// include uart function library
#include "rprintf.h"	// include printf function library
#include "timer.h"		// include timer function library (timing, PWM, etc)
#include "vt100.h"		// include VT100 terminal commands
#include "spi.h"		// include SPI interface functions
#include "mmc.h"		// include MMC card access functions
#include "debug.h"		// include debug function library
#include "fat.h"		// include fat16 functions


void mmcTest(void);	// function prototype

extern union FAT16_Buffer F16;

//############################################################################
// Begin Code 
int main(void)
//############################################################################
{
	// initialize libraries
	// initialize the UART (serial port)
	uartInit();
	//uartSetBaudRate(115200);
    UCSR0A |= (1<<U2X0);   // setup 115200 baud
	UBRRL=8;
	// make all rprintf statements use uart for output
	rprintfInit(uartSendByte);
	// initialize the timer system
	timerInit();
	// initialize vt100 terminal
	vt100Init();

	timerPause(100);

	// print welcome message
	vt100ClearScreen();
	vt100SetCursorPos(1,0);
	rprintfProgStrM("\r\nWelcome to the MMC Test Suite!\r\n");

	timerPause(1000);

	mmcTest();

	return 0;
}



//############################################################################
void mmcTest(void)
//############################################################################
{
	uint32_t sector=0;
	char FilePathName[64];
	int c;
	uint16_t i;
	
	// initialize MMC interface
	mmcInit();

	// print new prompt
	rprintf("\r\ncmd>");

	// testing loop
	while(1)
	{
		// check for keypress
		if((c=uartGetByte()) != -1)
		{
			rprintfChar(c);	rprintfCRLF();
			
			switch(c)
			{
			case 'i':
				// initialize card 
				rprintf("\r\nResetting MMC/SD Card\r\n");
				mmcReset();
				fatInit();
				break;
			case 'r':
				// read current sector into buffer
				//rprintf("\r\nRead Sector %d\r\n", sector);
				rprintfProgStrM("\r\nRead Sector ");	rprintfNum(10,10, FALSE,' ',sector);  rprintfCRLF();
				mmcRead(sector, (uint8_t*)(&F16.FATBuffer));
				// print buffer contents
				debugPrintHexTable(0x200,(uint8_t*)(&F16.FATBuffer));
				break;
			case 'w':
				// write current sector with data from buffer
				//rprintf("\r\nWrite Sector %d\r\n", sector);
				rprintfProgStrM("\r\nRead Sector ");	rprintfNum(10,10, FALSE,' ',sector);  rprintfCRLF();
				mmcWrite( sector, (uint8_t*)(&F16.FATBuffer) );
				break;
			// move to new sector
			case '+': sector++;		rprintfProgStrM("\r\nRead Sector ");	rprintfNum(10,10, FALSE,' ',sector);  rprintfCRLF(); break;
			case '-': sector--;		rprintfProgStrM("\r\nRead Sector ");	rprintfNum(10,10, FALSE,' ',sector);  rprintfCRLF(); break;
			case '*': sector+=0x200;	rprintfProgStrM("\r\nRead Sector ");	rprintfNum(10,10, FALSE,' ',sector);  rprintfCRLF(); break;
			case '_': sector-=0x200;	rprintfProgStrM("\r\nRead Sector ");	rprintfNum(10,10, FALSE,' ',sector);  rprintfCRLF(); break;
			case '\r':
			case 'g': fatInit(); break;
			case 'y': 
			{
				strcpy( FilePathName, "Neuer Ordner 123ABC/Neu Textdokument.txt");
				FAT16_Search( FilePathName , CONTINUE );
				break;
			}
			case 'x': 
			{
				strcpy( FilePathName, "A/B/C/test.hex");
				//FAT16_Dir( FilePathName , CONTINUE );
				FAT16_Search( FilePathName , OVERWRITE );
				break;
			}	
			case 'c': 
			{
				strcpy( FilePathName, "A/LangerName/A He/test.hex");
				FAT16_Search( FilePathName , OVERWRITE );
				break;
			}
			case 'v': 
			{
				strcpy( FilePathName, "A/LangerName/A He/test.hex");
				FAT16_Search( FilePathName , CONTINUE );
				break;
			}
			case 'b': 
			{
				strcpy( FilePathName, "testdokument.hex");
				FAT16_Search( FilePathName , CONTINUE );
				break;
			}	
			case 't':
			{ 	// TEST: write data in selected file
				for( i=0; i<400; i++)
				{
					FAT16_Write_Data( (uint32_t)i );
				}
				// data has to be finished before powering of mmc card
				// else filesize won't be saved and thus data can't 
				// be read by PC
				FAT16_Finish_Data();
				break;
			}  
			default:
				break;
			}
			// print new prompt
			rprintf("\r\ncmd> "); 
		}
	}

}

